﻿using Cinch;
using FR.SAFTAnalyser.Model;
using FR.SAFTAnalyser.ViewModels.Pane;
using MEFedMVVM.Common;
using OfficeOpenXml;
using OfficeOpenXml.Table;
using System;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FR.SAFTAnalyser.ViewModels
{
    public class MovementOfGoodsViewModel : DocumentPartViewModel
    {
        public MovementOfGoodsViewModel()
            : base("Mercadorias")
        {
            if (!Designer.IsInDesignMode)
            {
                Mediator.Instance.Register(this);

                DoOpenExcelCommand = new SimpleCommand<object, object>(CanOpenExcel, OpenExcel);
                ShowCustomerCommand = new SimpleCommand<object, object>(CanShowCustomer, DoShowCustomer);
                TestHashCommand = new SimpleCommand<object, object>(CanTestHash, DoTestHash);

                this.filtroDataInicio = DateTime.MinValue;
                this.FiltroDataFim = DateTime.MaxValue;
            }
        }

        internal override void ResetSaftFile()
        {
            this.StockMovement = null;
            this.CurrentMovement = null;
        }

        #region Mediator messages
        [MediatorMessageSink(MessageType.SAFT_FILE_OPENED)]
        private void Load(AuditFile saft)
        {
            if (saft != null && saft.SourceDocuments != null && saft.SourceDocuments.MovementOfGoods != null)
                this.MovementOfGoods = saft.SourceDocuments.MovementOfGoods;
        }

        [MediatorMessageSink(MessageType.INVOICE_LIST_CHANGED)]
        private void Reload(AuditFile saft)
        {
            if (saft != null && saft.SourceDocuments != null && saft.SourceDocuments.MovementOfGoods != null)
                this.MovementOfGoods = saft.SourceDocuments.MovementOfGoods;
        }
        #endregion

        #region Binding properties
        SourceDocumentsMovementOfGoods movementOfGoods;
        public SourceDocumentsMovementOfGoods MovementOfGoods
        {
            get
            {
                if (movementOfGoods == null && SAFT.SaftAnalyser.Instance.SaftFile != null && SAFT.SaftAnalyser.Instance.SaftFile.SourceDocuments != null)
                    movementOfGoods = SAFT.SaftAnalyser.Instance.SaftFile.SourceDocuments.MovementOfGoods;

                return movementOfGoods;
            }
            set
            {
                movementOfGoods = value;
                NotifyPropertyChanged(ObservableHelper.CreateArgs<MovementOfGoodsViewModel>(x => x.MovementOfGoods));

                if (movementOfGoods != null && string.IsNullOrEmpty(this.FilterMovement))
                {
                    DocNumberOfEntries = movementOfGoods.StockMovement.Length;
                    DocTotalQuantityIssued = movementOfGoods.StockMovement
                                            .Where(c => c.DocumentStatus.MovementStatus != MovementStatus.A)
                                            .Sum(c => c.Line.Sum(l => l.Quantity));

                    NumberOfEntries = movementOfGoods.NumberOfMovementLines;
                    TotalQuantityIssued = movementOfGoods.TotalQuantityIssued;

                    NotifyPropertyChanged(ObservableHelper.CreateArgs<MovementOfGoodsViewModel>(x => x.DocNumberOfEntries));
                    NotifyPropertyChanged(ObservableHelper.CreateArgs<MovementOfGoodsViewModel>(x => x.DocTotalQuantityIssued));

                    NotifyPropertyChanged(ObservableHelper.CreateArgs<MovementOfGoodsViewModel>(x => x.NumberOfEntries));
                    NotifyPropertyChanged(ObservableHelper.CreateArgs<MovementOfGoodsViewModel>(x => x.TotalQuantityIssued));

                    this.filtroDataInicio = movementOfGoods.StockMovement.Min(i => i.MovementDate);
                    this.filtroDataFim = movementOfGoods.StockMovement.Max(i => i.MovementDate);

                    this.StockMovement = movementOfGoods.StockMovement;
                }
            }
        }

        SourceDocumentsMovementOfGoodsStockMovement[] stockMovement;
        public SourceDocumentsMovementOfGoodsStockMovement[] StockMovement
        {
            get { return stockMovement; }
            set
            {
                stockMovement = value;
                NotifyPropertyChanged(ObservableHelper.CreateArgs<MovementOfGoodsViewModel>(x => x.StockMovement));
            }
        }

        SourceDocumentsMovementOfGoodsStockMovement currentMovement;
        public SourceDocumentsMovementOfGoodsStockMovement CurrentMovement
        {
            get { return currentMovement; }
            set
            {
                currentMovement = value;
                NotifyPropertyChanged(ObservableHelper.CreateArgs<MovementOfGoodsViewModel>(x => x.CurrentMovement));

                if (this.currentMovement != null)
                {
                    DocGrossTotal = currentMovement.Line.Sum(c => c.Item * (1 + c.Tax.TaxPercentage * 0.01m)).ToString("c");
                    DocNetTotal = currentMovement.Line.Sum(c => c.Item).ToString("c");
                    DocTaxPayable = currentMovement.Line.Sum(c => c.Item * c.Tax.TaxPercentage * 0.01m).ToString("c");

                    GrossTotal = currentMovement.DocumentTotals.GrossTotal.ToString("c");
                    NetTotal = currentMovement.DocumentTotals.NetTotal.ToString("c");
                    TaxPayable = currentMovement.DocumentTotals.TaxPayable.ToString("c");

                    NotifyPropertyChanged(ObservableHelper.CreateArgs<MovementOfGoodsViewModel>(x => x.DocGrossTotal));
                    NotifyPropertyChanged(ObservableHelper.CreateArgs<MovementOfGoodsViewModel>(x => x.DocNetTotal));
                    NotifyPropertyChanged(ObservableHelper.CreateArgs<MovementOfGoodsViewModel>(x => x.DocTaxPayable));

                    NotifyPropertyChanged(ObservableHelper.CreateArgs<MovementOfGoodsViewModel>(x => x.GrossTotal));
                    NotifyPropertyChanged(ObservableHelper.CreateArgs<MovementOfGoodsViewModel>(x => x.NetTotal));
                    NotifyPropertyChanged(ObservableHelper.CreateArgs<MovementOfGoodsViewModel>(x => x.TaxPayable));

                    if (string.IsNullOrEmpty(this.FilterLines) && !this.ShowAllLines)
                        this.CurrentMovementLines = currentMovement.Line;
                }
            }
        }

        SourceDocumentsMovementOfGoodsStockMovementLine[] currentMovementLines;
        public SourceDocumentsMovementOfGoodsStockMovementLine[] CurrentMovementLines
        {
            get { return currentMovementLines; }
            set
            {
                currentMovementLines = value;
                NotifyPropertyChanged(ObservableHelper.CreateArgs<MovementOfGoodsViewModel>(x => x.CurrentMovementLines));
            }
        }

        SourceDocumentsMovementOfGoodsStockMovementLine currentMovementLine;
        public SourceDocumentsMovementOfGoodsStockMovementLine CurrentMovementLine
        {
            get { return currentMovementLine; }
            set
            {
                currentMovementLine = value;
                NotifyPropertyChanged(ObservableHelper.CreateArgs<MovementOfGoodsViewModel>(x => x.CurrentMovementLine));
            }
        }

        bool showAllLines;
        public bool ShowAllLines
        {
            get { return showAllLines; }
            set
            {
                showAllLines = value;
                NotifyPropertyChanged(ObservableHelper.CreateArgs<MovementOfGoodsViewModel>(x => x.ShowAllLines));

                if (showAllLines)
                    this.CurrentMovementLines = StockMovement.SelectMany(i => i.Line).ToArray();
                else
                    this.CurrentMovementLines = CurrentMovement.Line;
            }
        }

        DateTime filtroDataInicio;
        public DateTime FiltroDataInicio
        {
            get { return filtroDataInicio; }
            set
            {
                filtroDataInicio = value;
                FilterMovementByDate();
                NotifyPropertyChanged(ObservableHelper.CreateArgs<MovementOfGoodsViewModel>(x => x.FiltroDataInicio));
            }
        }
        DateTime filtroDataFim;
        public DateTime FiltroDataFim
        {
            get { return filtroDataFim; }
            set
            {
                filtroDataFim = value;
                FilterMovementByDate();
                NotifyPropertyChanged(ObservableHelper.CreateArgs<MovementOfGoodsViewModel>(x => x.FiltroDataFim));
            }
        }

        private void FilterMovementByDate()
        {
            if (SAFT.SaftAnalyser.Instance.SaftFile != null && SAFT.SaftAnalyser.Instance.SaftFile.SourceDocuments.MovementOfGoods != null)
            {
                var inv = from i in SAFT.SaftAnalyser.Instance.SaftFile.SourceDocuments.MovementOfGoods.StockMovement
                          where i.MovementDate >= filtroDataInicio && i.MovementDate <= FiltroDataFim
                          select i;

                this.StockMovement = inv.ToArray();
            }
        }

        string filterMovement;
        public string FilterMovement
        {
            get { return filterMovement; }
            set
            {
                filterMovement = value;

                if (string.IsNullOrEmpty(filterMovement))
                    this.StockMovement = SAFT.SaftAnalyser.Instance.SaftFile.SourceDocuments.MovementOfGoods.StockMovement;
                else
                {
                    var inv = from i in SAFT.SaftAnalyser.Instance.SaftFile.SourceDocuments.MovementOfGoods.StockMovement
                              where i.DocumentNumber.IndexOf(filterMovement, StringComparison.OrdinalIgnoreCase) >= 0 ||
                              i.ATDocCodeID.IndexOf(filterMovement, StringComparison.OrdinalIgnoreCase) >= 0
                              select i;

                    this.StockMovement = inv.ToArray();
                }
            }
        }

        string filterLines;
        public string FilterLines
        {
            get { return filterLines; }
            set
            {
                filterLines = value;

                if (string.IsNullOrEmpty(filterLines))
                    this.CurrentMovementLines = this.CurrentMovement.Line;
                else
                {
                    var lines = from l in this.CurrentMovement.Line
                                where l.ProductDescription.IndexOf(filterLines, StringComparison.OrdinalIgnoreCase) >= 0 ||
                                        l.ProductCode.IndexOf(filterLines, StringComparison.OrdinalIgnoreCase) >= 0 ||
                                        l.Description.IndexOf(filterLines, StringComparison.OrdinalIgnoreCase) >= 0
                                select l;

                    this.CurrentMovementLines = lines.ToArray();
                }
            }
        }

        public string DocGrossTotal { get; set; }
        public string DocNetTotal { get; set; }
        public string DocTaxPayable { get; set; }
        public int DocNumberOfEntries { get; set; }
        public decimal DocTotalQuantityIssued { get; set; }

        public string NumberOfEntries { get; set; }
        public decimal TotalQuantityIssued { get; set; }
        public string GrossTotal { get; set; }
        public string NetTotal { get; set; }
        public string TaxPayable { get; set; }
        #endregion

        #region Commands
        public SimpleCommand<object, object> TestHashCommand { get; set; }
        private void DoTestHash(object obj)
        {
            string hashDocumentoAnterior = null;

            //fazer parse do campo InvoiceNo, descobrir o nº do documento actual e obter o documento anterior
            string[] docNo = this.CurrentMovement.DocumentNumber.Split('/');
            if (docNo != null && docNo.Length == 2)
            {
                int num = 0;
                Int32.TryParse(docNo[1], out num);
                num = num - 1;

                if (num > 0)
                {
                    SourceDocumentsMovementOfGoodsStockMovement movement
                        = this.StockMovement.Where(i => i.DocumentNumber.IndexOf(string.Format("{0}/{1}", docNo[0], num), StringComparison.OrdinalIgnoreCase) == 0).FirstOrDefault();

                    //encontramos um documento, vamos obter a hash
                    if (movement != null)
                        hashDocumentoAnterior = movement.Hash;
                }
            }

            Workspace.Workspace.Instance.ShowCreateHashTool(this.CurrentMovement, hashDocumentoAnterior);
        }
        private bool CanTestHash(object arg)
        {
            return this.CurrentMovement != null;
        }

        public SimpleCommand<object, object> DoOpenExcelCommand { get; private set; }
        private void OpenExcel(object args)
        {
            OpenMovementInExcel(this.CurrentMovement, this.ShowAllLines ? this.MovementOfGoods.StockMovement.SelectMany(m => m.Line).ToArray() : this.CurrentMovement.Line);
        }

        private bool CanOpenExcel(object args)
        {
            if (CurrentMovement != null && this.CurrentMovement.Line != null)
                return true;
            else
                return false;
        }

        public SimpleCommand<object, object> ShowCustomerCommand { get; private set; }
        private void DoShowCustomer(object args)
        {
            Workspace.Workspace.Instance.ShowCustomer(this.CurrentMovement.Pk);
        }
        private bool CanShowCustomer(object args)
        {
            return this.CurrentMovement != null;
        }
        #endregion

        private int Operation(SourceDocumentsMovementOfGoodsStockMovement i, SourceDocumentsMovementOfGoodsStockMovementLine l)
        {
            return 1;
        }

        private void OpenMovementInExcel(SourceDocumentsMovementOfGoodsStockMovement doc, SourceDocumentsMovementOfGoodsStockMovementLine[] lines)
        {
            if (doc != null && doc.DocumentStatus != null && doc.DocumentTotals != null)
            {
                DataTable dtinvoice = new DataTable("Movement");
                dtinvoice.Columns.Add("No", typeof(string));
                dtinvoice.Columns.Add("Estado", typeof(string));
                dtinvoice.Columns.Add("Data documento", typeof(DateTime));
                dtinvoice.Columns.Add("Data sistema", typeof(DateTime));
                dtinvoice.Columns.Add("Total s/ IVA", typeof(decimal));
                dtinvoice.Columns.Add("Total", typeof(decimal));
                dtinvoice.Columns.Add("Total imposto", typeof(decimal));
                dtinvoice.Columns.Add("Inicio transporte", typeof(DateTime));
                dtinvoice.Columns.Add("Fim transporte", typeof(DateTime));
                dtinvoice.Columns.Add("Periodo", typeof(int));
                dtinvoice.Columns.Add("Código CAE", typeof(string));
                dtinvoice.Columns.Add("Código", typeof(string));
                dtinvoice.Columns.Add("Data Estado", typeof(DateTime));
                dtinvoice.Columns.Add("Motivo", typeof(string));
                dtinvoice.Columns.Add("Origem", typeof(string));
                dtinvoice.Columns.Add("Utilizador (responsável pelo estado)", typeof(string));
                dtinvoice.Columns.Add("Utilizador (gerou o documento)", typeof(string));
                dtinvoice.Columns.Add("Transasção", typeof(string));
                dtinvoice.Columns.Add("Hash", typeof(string));
                dtinvoice.Columns.Add("Hash Test", typeof(string));
                dtinvoice.Columns.Add("Hash Control", typeof(string));

                dtinvoice.Rows.Add(
                    doc.DocumentNumber,
                    doc.DocumentStatus.MovementStatus.ToString(),
                    doc.MovementDate,
                    doc.SystemEntryDate,
                    doc.DocumentTotals.NetTotal,
                    doc.DocumentTotals.GrossTotal,
                    doc.DocumentTotals.TaxPayable,
                    doc.MovementStartTime,
                    doc.MovementEndTime,
                    Int32.Parse(doc.Period),
                    doc.EACCode,
                    doc.ATDocCodeID,
                    doc.DocumentStatus.MovementStatusDate,
                    doc.DocumentStatus.Reason,
                    doc.DocumentStatus.SourceBilling,
                    doc.DocumentStatus.SourceID,
                    doc.SourceID,
                    doc.TransactionID,
                    doc.Hash,
                    doc.HashTest,
                    doc.HashControl);

                ExcelPackage pck = new ExcelPackage();
                var wsEnum = pck.Workbook.Worksheets.Add("SAFT Movement");
                wsEnum.Cells["C2"].Style.Numberformat.Format = "yyyy-MM-dd";
                wsEnum.Cells["D2"].Style.Numberformat.Format = "yyyy-MM-dd hh:mm:ss";
                wsEnum.Cells["H2"].Style.Numberformat.Format = "yyyy-MM-dd hh:mm:ss";
                wsEnum.Cells["I2"].Style.Numberformat.Format = "yyyy-MM-dd hh:mm:ss";
                wsEnum.Cells["M2"].Style.Numberformat.Format = "yyyy-MM-dd hh:mm:ss";

                wsEnum.Cells["E2"].Style.Numberformat.Format = @"#,##0.00";
                wsEnum.Cells["F2"].Style.Numberformat.Format = @"#,##0.00";
                wsEnum.Cells["G2"].Style.Numberformat.Format = @"#,##0.00";
                //Load the collection starting from cell A1...
                wsEnum.Cells["A1"].LoadFromDataTable(dtinvoice, true, TableStyles.Medium9);
                wsEnum.Cells[wsEnum.Dimension.Address].AutoFitColumns();

                if (doc.Line != null)
                {
                    DataTable dtlines = new DataTable("MovementLines");

                    dtlines.Columns.Add("Nº linha", typeof(int));
                    dtlines.Columns.Add("Código produto", typeof(string));
                    dtlines.Columns.Add("Descrição produto", typeof(string));
                    dtlines.Columns.Add("Quantidade", typeof(decimal));
                    dtlines.Columns.Add("Preço unitário", typeof(decimal));
                    dtlines.Columns.Add("Valor", typeof(decimal));
                    dtlines.Columns.Add("Desconto", typeof(decimal));
                    dtlines.Columns.Add("Tipo operação", typeof(string));
                    dtlines.Columns.Add("Imposto", typeof(decimal));
                    dtlines.Columns.Add("Isenção imposto", typeof(string));
                    dtlines.Columns.Add("Unidade medida", typeof(string));
                    dtlines.Columns.Add("Descrição", typeof(string));
                    dtlines.Columns.Add("DocNumber", typeof(string));

                    foreach (var line in lines)
                    {
                        dtlines.Rows.Add(
                            line.LineNumber,
                            line.ProductCode,
                            line.ProductDescription,
                            line.Quantity,
                            line.UnitPrice,
                            line.Item,
                            line.SettlementAmount,
                            line.ItemElementName.ToString(),
                            line.Tax.TaxPercentage,
                            line.TaxExemptionReason,
                            line.UnitOfMeasure,
                            line.Description,
                            line.DocNo);
                    }

                    var wsEnumLines = pck.Workbook.Worksheets.Add("SAFT Movement lines");

                    wsEnumLines.Cells[string.Format("D2:D{0}", lines.Length + 1)].Style.Numberformat.Format = @"#,##0.00";
                    wsEnumLines.Cells[string.Format("E2:E{0}", lines.Length + 1)].Style.Numberformat.Format = @"#,##0.00";
                    wsEnumLines.Cells[string.Format("F2:F{0}", lines.Length + 1)].Style.Numberformat.Format = @"#,##0.00";
                    wsEnumLines.Cells[string.Format("G2:G{0}", lines.Length + 1)].Style.Numberformat.Format = @"#,##0.00";

                    wsEnumLines.Cells[string.Format("D{0}", lines.Length + 2)].Formula = string.Format("SUM(D2:D{0})", lines.Length + 1);
                    wsEnumLines.Cells[string.Format("E{0}", lines.Length + 2)].Formula = string.Format("SUM(E2:E{0})", lines.Length + 1);
                    wsEnumLines.Cells[string.Format("F{0}", lines.Length + 2)].Formula = string.Format("SUM(F2:F{0})", lines.Length + 1);
                    wsEnumLines.Cells[string.Format("G{0}", lines.Length + 2)].Formula = string.Format("SUM(G2:G{0})", lines.Length + 1);

                    //Load the collection starting from cell A1...
                    wsEnumLines.Cells["A1"].LoadFromDataTable(dtlines, true, TableStyles.Medium9);
                    wsEnumLines.Cells[wsEnum.Dimension.Address].AutoFitColumns();
                }

                //...and save
                var fi = new FileInfo(Path.GetTempPath() + Path.GetRandomFileName() + ".xlsx");

                pck.SaveAs(fi);

                Process.Start(fi.FullName);
            }
        }
    }
}
