﻿using Cinch;
using FR.SAFTAnalyser.ViewModels.Pane;
using MEFedMVVM.Common;
using MEFedMVVM.ViewModelLocator;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FR.SAFTAnalyser.ViewModels.Tools
{
    public class SaftPartsViewModel : ToolViewModel, ITool
    {
        public SaftPartsViewModel()
            : base("Tabelas")
        {
            if (!Designer.IsInDesignMode)
                DoOpenSaftPartCommand = new SimpleCommand<object, object>(CanOpenSaftPart, OpenSaftPart);
        }

        SaftPartDetails currentSaftPart;
        public SaftPartDetails CurrentSaftPart
        {
            get { return currentSaftPart; }
            set
            {
                currentSaftPart = value;
                NotifyPropertyChanged(Cinch.ObservableHelper.CreateArgs<SaftPartsViewModel>(x => x.CurrentSaftPart));
            }
        }

        ObservableCollection<SaftPartDetails> saftParts;
        public ObservableCollection<SaftPartDetails> SaftParts
        {
            get
            {
                if (saftParts == null)
                {
                    saftParts = new ObservableCollection<SaftPartDetails>();

                    saftParts.Add(new SaftPartDetails { Name = SaftPart.Invoices, Description = "Documentos comerciais" });
                    saftParts.Add(new SaftPartDetails { Name = SaftPart.MovementOfGoods, Description = "Mercadorias" });
                    saftParts.Add(new SaftPartDetails { Name = SaftPart.Payments, Description = "Recibos" });
                    saftParts.Add(new SaftPartDetails { Name = SaftPart.WorkingDocuments, Description = "Documentos conferência" });
                    saftParts.Add(new SaftPartDetails { Name = SaftPart.GeneralLedgerEntries, Description = "Movimentos contabilísticos" });
                    saftParts.Add(new SaftPartDetails { Name = SaftPart.GeneralLedger, Description = "Planos contas" });
                    saftParts.Add(new SaftPartDetails { Name = SaftPart.Customers, Description = "Clientes" });
                    saftParts.Add(new SaftPartDetails { Name = SaftPart.Supplier, Description = "Fornecedores" });
                    saftParts.Add(new SaftPartDetails { Name = SaftPart.Products, Description = "Produtos" });
                    saftParts.Add(new SaftPartDetails { Name = SaftPart.Tax, Description = "Impostos" });
                }

                return saftParts;
            }
            set
            {
                saftParts = value;
                NotifyPropertyChanged(Cinch.ObservableHelper.CreateArgs<SaftPartsViewModel>(x => x.SaftParts));
            }
        }


        PaneLocation preferredLocation = new PaneLocation { Location = PaneLocation.LEFT };
        public PaneLocation PreferredLocation
        {
            get { return preferredLocation; }
            set { preferredLocation = value; }
        }

        bool isAutoHidden = true;
        public bool IsAutoHidden
        {
            get { return isAutoHidden; }
            set { isAutoHidden = value; }
        }

        public SimpleCommand<object, object> DoOpenSaftPartCommand { get; set; }

        private void OpenSaftPart(object obj)
        {
            EventToCommandArgs commandArgs = obj as EventToCommandArgs;
            if (commandArgs != null && commandArgs.CommandParameter != null)
                Workspace.Workspace.Instance.ShowSaftPart(((SaftPartDetails)commandArgs.CommandParameter).Name);
        }

        private bool CanOpenSaftPart(object arg)
        {
            return SAFTAnalyser.SAFT.SaftAnalyser.Instance.SaftFile != null;
        }
    }

    public struct SaftPartDetails
    {
        public SaftPart Name { get; set; }
        public string Description { get; set; }
        public string ImageSource { get; set; }
    }

    public enum SaftPart
    {
        Invoices,
        MovementOfGoods,
        Payments,
        WorkingDocuments,
        InvoicesSummary,
        GeneralLedgerEntries,
        GeneralLedger,
        Customers,
        Supplier,
        Products,
        Tax,
        ConvertPemKey
    }
}
