﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.Serialization;

namespace FR.SAFTAnalyser.ViewModels.Workspace
{
    [Serializable]
    [XmlRoot(ElementName = "ProgramSettings", IsNullable = false)]
    public class ConfigViewModel : INotifyPropertyChanged
    {
        #region constructor
        public ConfigViewModel()
        {
            this.RememberLastOpenFiles = true;
            this.MinDate = new DateTime(2010, 01, 01);
            this.NumCasasDecimaisValidacoes = 2;
            this.FontSize = 14;
            this.ConnectionString = @"data source=;initial catalog=;timeout=200;user id=;password=;";
            this.OpenedTables = CreateDefaultOpenedTables();
        }
        #endregion constructor

        #region properties

        public string SAFTFileName { get; set; }

        public string SchemaVersion { get; set; }

        public bool RememberLastOpenFiles { get; set; }

        public string PrivateKeyFileName { get; set; }

        public DateTime MinDate { get; set; }

        public int FontSize { get; set; }

        public int NumCasasDecimaisValidacoes { get; set; }

        public string PublicKeyFileName { get; set; }

        SaftTablesDescription[] openedTables;
        public SaftTablesDescription[] OpenedTables
        {
            get
            {
                if (openedTables == null) openedTables = CreateDefaultOpenedTables();
                return openedTables;
            }
            set { if (value != null) openedTables = value; }
        }

        /// <summary>
        /// The avalonDock theme
        /// </summary>
        public int ThemeID { get; set; }

        public string ConnectionString { get; set; }

        #endregion properties

        #region methods
        #region Load Save ProgramOptions ViewModel
        /// <summary>
        /// Save program options into persistence.
        /// See <seealso cref="SaveOptions"/> to save program options on program end.
        /// </summary>
        /// <param name="settingsFileName"></param>
        /// <returns></returns>
        public static ConfigViewModel LoadOptions(string settingsFileName)
        {
            ConfigViewModel loadedViewModel = null;

            if (System.IO.File.Exists(settingsFileName))
            {
                try
                {
                    loadedViewModel = FR.SAFTAnalyser.SAFT.XmlSerializer.Deserialize<ConfigViewModel>(settingsFileName);
                }
                catch
                {
                    loadedViewModel = new ConfigViewModel();
                }
            }

            return loadedViewModel;
        }

        /// <summary>
        /// Save program options into persistence.
        /// See <seealso cref="LoadOptions"/> to load program options on program start.
        /// </summary>
        /// <param name="settingsFileName"></param>
        /// <param name="vm"></param>
        /// <returns></returns>
        public static bool SaveOptions(string settingsFileName, ConfigViewModel vm)
        {
            return FR.SAFTAnalyser.SAFT.XmlSerializer.Serializa(vm, settingsFileName);
        }
        #endregion Load Save ProgramOptions ViewModel

        internal SaftTablesDescription[] CreateDefaultOpenedTables()
        {
            return new SaftTablesDescription[] 
            {
                new SaftTablesDescription { Table = SaftTables.Customers, IsOpenByDefault = true, Description = "Clientes"},
                new SaftTablesDescription { Table = SaftTables.GeneralLeader, IsOpenByDefault = true, Description = "Planos contas"},
                new SaftTablesDescription { Table = SaftTables.GeneralLeaderEntries, IsOpenByDefault = true, Description = "Movimentos"},
                new SaftTablesDescription { Table = SaftTables.Header, IsOpenByDefault = true, Description = "Cabeçalho"},
                new SaftTablesDescription { Table = SaftTables.Invoices, IsOpenByDefault = true, Description = "Documentos comerciais"},
                new SaftTablesDescription { Table = SaftTables.MovementOfGoods, IsOpenByDefault = true, Description = "Mercadorias"},
                new SaftTablesDescription { Table = SaftTables.Payments, IsOpenByDefault = true, Description = "Recibos"},
                new SaftTablesDescription { Table = SaftTables.Products, IsOpenByDefault = true, Description = "Produtos"},
                new SaftTablesDescription { Table = SaftTables.Suppliers, IsOpenByDefault = true, Description = "Fornecedores"},
                new SaftTablesDescription { Table = SaftTables.Taxes, IsOpenByDefault = true, Description = "Impostos"},
                new SaftTablesDescription { Table = SaftTables.WorkingDocuments, IsOpenByDefault = true, Description = "Documentos de conferência"}
            };
        }

        #endregion methods

        public event PropertyChangedEventHandler PropertyChanged;
        protected void NotifyPropertyChanged(String propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;

            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
