﻿namespace FR.SAFTAnalyser.ViewModels.Workspace
{
    using FR.SAFTAnalyser.Services;
    using FR.SAFTAnalyser.ViewModels.Base;
    using System;

    internal partial class Workspace
    {
        private ConfigViewModel _config = null;

        /// <summary>
        /// Get/set program settings for entire application
        /// </summary>
        public ConfigViewModel Config
        {
            get
            {
                if (_config == null)
                    _config = new ConfigViewModel();

                return _config;
            }
            internal set
            {
                if (_config != value)
                {
                    _config = value;
                    NotifyPropertyChanged(ObservableHelper.CreateArgs<Workspace>(x => x.Config));
                }
            }
        }

        /// <summary>
        /// Save application settings when the application is being closed down
        /// </summary>
        public void SaveConfigOnAppClosed()
        {
            try
            {
                App.CreateAppDataFolder();

                // Save/initialize program options that determine global program behaviour
                ConfigViewModel.SaveOptions(App.DirFileAppSessionData, this.Config);
            }
            catch (Exception exp)
            {
                ServiceResolver.Instance.MessageBoxService.ShowError(exp.ToString(), "Unhandled Exception");
            }
        }

        /// <summary>
        /// Load configuration from persistence on startup of application
        /// </summary>
        public void LoadConfigOnAppStartup()
        {
            ConfigViewModel retOpts = null;

            try
            {
                // Re/Load program options to control global behaviour of program
                if ((retOpts = ConfigViewModel.LoadOptions(App.DirFileAppSessionData)) == null)
                    retOpts = new ConfigViewModel();
            }
            catch { }
            finally
            {
                if (retOpts == null)
                    retOpts = new ConfigViewModel();
            }

            this.Config = retOpts;
        }
    }
}
