﻿using Cinch;
using System.Windows;
using System.Windows.Controls;
using System.Linq;
using System.Data;
using OfficeOpenXml;
using OfficeOpenXml.Table;
using System.IO;
using System.Diagnostics;
using System;
using System.Collections.Generic;
using FR.SAFTAnalyser.Model;
using System.Collections;

namespace FR.SAFTAnalyser.ViewModels
{
    public class WorkspaceViewModel : ViewModelBase
    {
        protected WorkspaceViewModel()
        {
            //Commands
            DoPrintCommand = new SimpleCommand<object, object>(CanPrint, Print);
        }

        public SimpleCommand<object, object> DoPrintCommand { get; private set; }
        private void Print(object args)
        {
            DataGrid datagrid = args as DataGrid;
            if (datagrid != null)
            {
                if (datagrid.ItemsSource != null)
                {
                    DataTable dt = new DataTable();
                    foreach (DataGridColumn column in datagrid.Columns)
                    {
                        if (column is DataGridTextColumn && column.Visibility == Visibility.Visible)
                            dt.Columns.Add(column.Header.ToString());
                    }

                    foreach (DataGridRow row in GetDataGridRows(datagrid))
                    {
                        List<object> rows = new List<object>();
                        foreach (DataGridColumn column in datagrid.Columns)
                        {
                            if (column is DataGridTextColumn)
                            {
                                TextBlock txt = column.GetCellContent(row) as TextBlock;
                                if (txt != null)
                                    rows.Add(txt.Text);
                                else
                                    rows.Add("");
                            }
                        }
                        dt.Rows.Add(rows.ToArray());
                    }

                    ExcelPackage pck = new ExcelPackage();
                    var wsEnum = pck.Workbook.Worksheets.Add("SAFT");
                    //Load the collection starting from cell A1...
                    wsEnum.Cells["A1"].LoadFromDataTable(dt, true, TableStyles.Medium9);
                    wsEnum.Cells[wsEnum.Dimension.Address].AutoFitColumns();

                    //...and save
                    var fi = new FileInfo(Path.GetTempPath() + Path.GetRandomFileName() + ".xlsx");
                    if (fi.Exists)
                        fi.Delete();
                    
                    pck.SaveAs(fi);

                    Process.Start(fi.FullName);
                }
            }
        }

        private bool CanPrint(object args)
        {
            DataGrid datagrid = args as DataGrid;
            if (datagrid != null)
            {
                if (datagrid.ItemsSource != null)
                    return true;
            }

            return false;
        }

        private IEnumerable<DataGridRow> GetDataGridRows(DataGrid grid)
        {
            var itemsSource = grid.ItemsSource as IEnumerable;
            if (null == itemsSource) yield return null;
            foreach (var item in itemsSource)
            {
                var row = grid.ItemContainerGenerator.ContainerFromItem(item) as DataGridRow;
                if (null != row) yield return row;
            }
        }
    }
}
